@@ -14,18 +14,38 @@ pub enum Move {
     Dont,
 }
 
-type Board = Vec<Vec<i32>>;
+/// Used to depict the status in the [`Game`] API
+pub enum Status {
+    /// Game has finished, player won
+    Won,
+    /// Game has finished, player lost
+    Lost,
+    /// Game continues, neither won nor lost
+    On,
+}
+
+type Board = Vec<Vec<usize>>;
 
 /// An object that models the board to play 2048 on and defines the rules for the game
 pub struct Game {
     board: Board,
     board_size: usize,
-    winning: i32,
+    winning: usize,
 }
 
 impl Game {
     /// Constructs a board to play the game
-    pub fn new(board_size: usize, winning: i32) -> Self {
+    /// board_size >= 2, defines board's length & breadth
+    /// winning >= 4 and a power of 2, defines the value for game to havae been won
+    pub fn new(board_size: usize, winning: usize) -> Self {
+        // Ensure the board size is atleast 2
+        let board_size = board_size.clamp(2, usize::MAX);
+        // Ensure the winning point is a power of two and >= 4, in order to be reached within game logic
+        let winning = 2usize
+            .pow((winning as f32).log2().abs() as u32)
+            .clamp(4, usize::MAX);
+
+        // initialize an empty board of 0s
         let empty = vec![0].repeat(board_size);
         let mut board = vec![];
         board.resize(board_size, empty);
@@ -47,6 +67,11 @@ impl Game {
         &self.board
     }
 
+    /// Returns the value for winning
+    pub fn winning(&self) -> usize {
+        self.winning
+    }
+
     /// Performs the compression of board's values towards the left most column
     fn move_left(&mut self) {
         for i in 0..self.board_size {
@@ -111,6 +136,7 @@ impl Game {
 
         loop {
             let x: usize = rng.gen();
+
             if self.board[x % self.board_size][(x / 10) % self.board_size] == 0 {
                 if x % 5 == 0 {
                     self.board[x % self.board_size][(x / 10) % self.board_size] = 4;
@@ -152,14 +178,25 @@ impl Game {
     }
 
     /// Check if board contains value x
-    fn contains(&self, x: i32) -> bool {
+    fn contains(&self, x: usize) -> bool {
         self.board
             .iter()
             .fold(false, |t, v| t || v.iter().fold(false, |u, w| u || *w == x))
     }
 
+    pub fn status(&self) -> Status {
+        if self.contains(self.winning) {
+            Status::Won
+        } else if self.is_locked() {
+            Status::Lost
+        } else {
+            Status::On
+        }
+    }
+
     /// [`Game`] API entry-point, operated by [`Move`] as input
-    pub fn mover(&mut self, mov: Move) -> Result<bool, String> {
+    /// Output bool is used to check if move caused any change to the board
+    pub fn mover(&mut self, mov: Move) -> bool {
         let temp = self.board.clone();
 
         match mov {
@@ -170,19 +207,11 @@ impl Game {
             _ => (),
         }
 
-        if self.is_locked() {
-            return Err("You have Lost!".to_string());
-        }
-
-        if self.contains(self.winning) {
-            return Err("You have Won!".to_string());
-        }
-
-        Ok(self.board != temp)
+        self.board != temp
     }
 
     /// Compress a row/column
-    fn vec_compress(&self, v: &mut Vec<i32>) {
+    fn vec_compress(&self, v: &mut Vec<usize>) {
         v.retain(|x| *x != 0);
         let vl = v.len();
 
